/* Revision: 0.0.0 */

/******************************************************************************
* Copyright 1998-2016 NetBurner, Inc.  ALL RIGHTS RESERVED
*
*    Permission is hereby granted to purchasers of NetBurner Hardware to use or
*    modify this computer program for any use as long as the resultant program
*    is only executed on NetBurner provided hardware.
*
*    No other rights to use this program or its derivatives in part or in
*    whole are granted.
*
*    It may be possible to license this or other NetBurner software for use on
*    non-NetBurner Hardware. Contact sales@Netburner.com for more information.
*
*    NetBurner makes no representation or warranties with respect to the
*    performance of this computer program, and specifically disclaims any
*    responsibility for any damages, special or consequential, connected with
*    the use of this program.
*
* NetBurner
* 5405 Morehouse Dr.
* San Diego, CA 92121
* www.netburner.com
******************************************************************************/

/*------------------------------------------------------------------------------
 * PINK application serial command test
 *
 * The example was written for and tested on a NetBurner MOD54415 core module.
 * The purpose is to send serial commands to a NetBurner module running the
 * PINK serial-enabled web server application code to test various functions.
 * The test was run on PINK application version 1.44.
 *
 * Hardware configuration:
 * - Both modules use UART1 as the data channel. The MOD54415 uses UAR0 for the
 *   status message and control port, so connect MTTTY to UART0.
 *
 *  MTTTY  <-- RS-232 --> MOD54415 <-- RS-232 --> SB70LC/PINK <-- Ethernet --> Your PC
 * (Your PC)
 *
 * Software configuration PINK:
 * - Connect module to your network, use web browser to get to configuration page
 * - Configure the UART1 data port to be 9600 baud (you can use baud rates
 *   up to 115,200, just make sure the baud rate matches for this application
 *   and the PINK).
 *
 *----------------------------------------------------------------------------*/
#include <predef.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <utils.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <serial.h>

#ifdef _DEBUG
#include <networkdebug.h>
#endif

#define SERIAL_DEBUG_PORT 0
#define SERIAL_DATA_PORT 1


const char *AppName = "PINK serial command test";

extern "C"
{
    void UserMain(void *pd);
}


int fdSerialData;


/*-------------------------------------------------------------------
 * Send a command to read a value
 *-----------------------------------------------------------------*/
int SendReadCommand(const char *command)
{
    char buf[80];
    int error = 0;

    writeall(fdSerialData, command, strlen(command));

    OSTimeDly(TICKS_PER_SECOND);    // Needed for slow baud rates (eg 2400)

    if (dataavail(fdSerialData))
    {
        int bytesRead = read(fdSerialData, buf, 80);
        if ( bytesRead > 0)
        {
            buf[bytesRead] = '\0';
            iprintf("%s", buf);
        }
        else
        {
            iprintf("Error: bytesRead = %d for command %s\r\n", bytesRead, command);
            error = -1;
        }
    }
    else
    {
        iprintf("Error: No data available to read from command %s\r\n", command);
        error = -2;
    }

    return error;
}


/*-------------------------------------------------------------------
 * Send a command to write a value
 *-----------------------------------------------------------------*/
void SendWriteCommand(const char *command)
{
    char buf[80];

    writeall(fdSerialData, command, strlen(command));
    buf[0] = 0;
    write(fdSerialData, buf, 1);   // write command terminator is 0
    OSTimeDly(TICKS_PER_SECOND / 5);
}



/*-------------------------------------------------------------------
 * Get system info from PINK modle
 *-----------------------------------------------------------------*/
int GetSystemInformation()
{
    int error = 0;

    const char cmd[][80] = {
        "!NB0RSI",
        "!NB0RSN",
        "!NB0RSG",
        "!NB0RSD",
        "!NB0RHP",
        "!NB0RFW",
        "!NB0RST",
        ""
    };

    int i = 0;

    iprintf("\r\n--- System Information ---\r\n");
    while ( cmd[i][0] != NULL)
    {
        iprintf("Sending: %s ", cmd[i]);
        error = SendReadCommand(cmd[i]);
        iprintf("\r\n");
        i++;
    }

    return error;
}


/*-------------------------------------------------------------------
 * Write data variables
 *-----------------------------------------------------------------*/
void WriteAllDataVariables()
{
    char buf[80];

    uint32_t secStart = Secs;
    iprintf("Writing vars 0 - 99\r\n");
    for ( int i = 0; i < 100; i++ )
    {
        siprintf(buf, "!NB0W%02d:%d", i, i);
        SendWriteCommand(buf);
        iprintf("%d\r", i);
    }
    iprintf("\r\ncomplete (approx. %ld seconds)\r\n", Secs - secStart);
}



/*-------------------------------------------------------------------
 * Get system info from PINK modle
 *-----------------------------------------------------------------*/
void GetSystemInformationContinuous( bool writeAllVars )
{
    int error = 0;
    uint32_t count = 0;

    while ( (error == 0) && (!charavail() ) )
    {
        iprintf("Count: %ld, System seconds: %ld\r\n", count, Secs);
        error = GetSystemInformation();

        if ( (error == 0) && (writeAllVars) )
            WriteAllDataVariables();
    }

    if ( charavail() )
    {
        iprintf("Test exited by user\r\n");
        char c = getchar();
    }

    if ( error != 0 )
        iprintf("*** Test terminated due to error\r\n");
}




/*-------------------------------------------------------------------
 * Set static ip settings
 *-----------------------------------------------------------------*/
void WriteStaticIpSettings()
{
    const char cmd[][80] = {
        "!NB0WSI:192.168.1.10",
        "!NB0WSN:255.255.255.0",
        "!NB0WSG:192.168.1.1",
        "!NB0WSD:8.8.8.8",
        "!NB0WHP:8080",
        ""
    };

    int i = 0;

    iprintf("\r\nSetting Static IP Configuration ---\r\n");
    while ( cmd[i][0] != NULL)
    {
        iprintf("Writing: %s\r\n", cmd[i]);
        SendWriteCommand(cmd[i]);
        OSTimeDly(TICKS_PER_SECOND);  // allow time for flash write for each parameter
        i++;
    }
}



/*-------------------------------------------------------------------
 * Set static ip settings
 *-----------------------------------------------------------------*/
void ClearStaticIpSettings()
{
    const char cmd[][80] = {
        "!NB0WSI:0.0.0.0",
        "!NB0WSN:0.0.0.0",
        "!NB0WSG:0.0.0.0",
        "!NB0WSD:0.0.0.0",
        "!NB0WHP:80",
        ""
    };

    int i = 0;

    iprintf("\r\nClearing Static IP Configuration ---\r\n");
    while ( cmd[i][0] != NULL)
    {
        iprintf("Writing: %s\r\n", cmd[i]);
        SendWriteCommand(cmd[i]);
        OSTimeDly(TICKS_PER_SECOND);    // allow time for flash write for each parameter
        i++;
    }
}


/*-------------------------------------------------------------------
 * Write HTTP port number
 *-----------------------------------------------------------------*/
void WriteHttpPortNumber()
{
    char buf[10];

    iprintf("Enter port number: ");
    gets(buf);
    if (strlen(buf) > 9)
    {
        iprintf("Invalid port number: %s", buf);
        return;
    }

    char cmd[80];
    sniprintf(cmd, 80, "!NB0WHP:%s", buf);
    iprintf("\r\nWriting: %s\r\n", cmd);
    SendWriteCommand(cmd);
    OSTimeDly(TICKS_PER_SECOND);    // allow time for flash write for each parameter
}



/*-----------------------------------------------------------------------------
 * Display debug menu
 *-----------------------------------------------------------------------------*/
void displayMenu(void)
{
    iprintf("\r\n----- Main Menu -----\r\n");
    iprintf("1 - Write Static IP settings\r\n");
    iprintf("2 - Clear Static IP settings\r\n");
    iprintf("3 - Write HTTP Port Number\r\n");
    iprintf("4 - Get PINK system status Continuous\r\n");
    iprintf("5 - Write all variables 0 - 99\r\n");
    iprintf("6 - Get PINK system status and write all vars Continuous\r\n");

    iprintf("R - Send Reset Command\r\n");
    iprintf("S - Get PINK system status\r\n");

}


/*-----------------------------------------------------------------------------
 * Handle commands with a simple command dispatcher
 *----------------------------------------------------------------------------*/
void processCommand(char *buffer)
{
    switch (toupper(buffer[0]))
    {
      case '1':
         WriteStaticIpSettings();
        break;

      case '2':
         ClearStaticIpSettings();
        break;

      case '3':
         WriteHttpPortNumber();
        break;

      case '4':
          GetSystemInformationContinuous( false );  // status only, not writing vars
          break;

      case '5':
          WriteAllDataVariables();
          break;

      case '6':
          GetSystemInformationContinuous( true );   // write all vars as well
          break;

      case 'R':
          iprintf("Resetting....\r\n");
          SendWriteCommand("!NB0WSR:0");
          break;

      case 'S':
          GetSystemInformation();
          break;

      default:
        displayMenu();
    }
}


/*-----------------------------------------------------------------------------
 * The main task
 *--------------------------------------------------------------------------------*/
void UserMain(void *pd)
{
    InitializeStack();

    if (EthernetIP.IsNull())
    {
        iprintf("Trying DHCP...\r\n");
        GetDHCPAddress();
        iprintf("DHCP assigned the IP address of:  ");
        ShowIP(EthernetIP);
        iprintf("\r\n");
    }
    else
    {
        iprintf("Static IP address:  ");
        ShowIP(EthernetIP);
        iprintf("\r\n");
    }

    OSChangePrio(MAIN_PRIO);   // Set main task priority to the default (50)
    EnableAutoUpdate();        // Enable NB application AutoUpdate support

#ifdef _DEBUG
    InitializeNetworkGDB();
    //InitializeNetworkGDB_and_Wait();
#endif // _DEBUG

    // You can change the baud rate here
    fdSerialData = SimpleOpenSerial(SERIAL_DATA_PORT, 9600);

    displayMenu();
    char buffer[255];
    buffer[0] = '\0';

    while (1)
    {
        if (charavail())
        {
            gets(buffer);
            processCommand(buffer);
            buffer[0] = '\0';
        }
    }
}
